import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:vaycore/dartin.dart';
import 'package:vaycore/mvvm.dart';
import 'package:vaycore/vaycore.dart';

///
/// @author vay
/// @email hxhvay@126.com
/// @date 2020-05-16
///

/// 页面的基类 [BasePage]
abstract class BasePage extends StatelessWidget {
  BasePage({Key key}) : super(key: key);

  Widget buildContent(BuildContext context);

  @override
  Widget build(BuildContext context) {
    if (core.context == null) {
      core.context = context;
    }
    return buildContent(context);
  }
}

/// 无状态页面基类 [BaseStatelessPage]
abstract class BaseStatelessPage<T extends BaseProvider> extends BasePage {
  final T mProvider;

  /// 构造函数
  ///
  /// [params] 代表注入ViewModel[mProvider]时所需的参数，需按照[mProvider]的构造方法顺序赋值
  BaseStatelessPage({Key key, List<dynamic> params})
      : mProvider = inject<T>(params: params),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider<T>.value(
      value: mProvider,
      child: super.build(context),
    );
  }
}

/// 有状态的页面
abstract class BaseStatefulPage<T extends BaseProvider> extends StatefulWidget {
  final T provider;

  /// 构造函数
  ///
  /// [params] 代表注入ViewModel[provider]时所需的参数，需按照[provider]的构造方法顺序赋值
  BaseStatefulPage({Key key, List<dynamic> params})
      : provider = inject<T>(params: params),
        super(key: key);

  BaseState<BaseStatefulPage<T>, T> createState() {
    BaseState state = buildState();
    state.provider = this.provider;
    return state;
  }

  /// 构建[BaseState]对象
  BaseState buildState();
}

/// 有状态页面的状态类
abstract class BaseState<T extends BaseStatefulPage, P extends BaseProvider>
    extends State<T> with AutomaticKeepAliveClientMixin {
  P provider;

  @override
  bool get wantKeepAlive => false;

  Widget buildContent(BuildContext context);

  @override
  @mustCallSuper
  void initState() {
    // 用于初始化一些变量
    provider.onInit();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    if (core.context == null) {
      core.context = context;
    }
    return ChangeNotifierProvider<P>.value(
      value: provider,
      child: buildContent(context),
    );
  }
}
