import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

///
/// @author vay
/// @email hxhvay@126.com
/// @date 2020-05-23
///

/// 加载中的布局（用于在界面数据未加载完成时显示）
class LoadingContainer extends StatelessWidget {
  /// 子组件
  final Widget child;

  /// 是否加载的是封面
  final bool cover;

  /// 加载状态
  final LoadingStatus status;

  /// 加载中显示的组件
  final Widget loading;

  /// 加载失败时显示的组件
  final Widget failed;

  /// 失败时点击重试
  final VoidCallback onRetry;

  LoadingContainer({
    Key key,
    @required this.child,
    @required this.status,
    this.cover = false,
    Widget loading,
    Widget failed,
    this.onRetry,
  })  : this.loading = loading ?? _defaultLoading(),
        this.failed = failed ?? _defaultLoadFailed(),
        super(key: key);

  /// 默认加载中控件
  static Widget _defaultLoading() {
    return Center(
      child: CupertinoActivityIndicator(),
    );
  }

  /// 默认加载失败控件
  static Widget _defaultLoadFailed() {
    return Center(
      child: Icon(Icons.error, size: 50),
    );
  }

  @override
  Widget build(BuildContext context) {
    if (!this.cover) {
      return _buildWidget();
    }
    return Stack(
      children: <Widget>[
        this.child,
        _buildCoverWidget(),
      ],
    );
  }

  /// 加载中组件
  Widget _buildWidget() {
    switch (this.status) {
      case LoadingStatus.loading:
        // 加载中
        return this.loading;
      case LoadingStatus.failed:
        // 加载失败
        return _buildLoadFailed();
      case LoadingStatus.success:
      default:
        // 加载成功
        return this.child;
    }
  }

  /// 封面图的加载中组件
  Widget _buildCoverWidget() {
    switch (this.status) {
      case LoadingStatus.loading:
        // 加载中
        return this.loading;
      case LoadingStatus.failed:
        // 加载失败
        return _buildLoadFailed();
      case LoadingStatus.success:
      default:
        // 加载成功
        return null;
    }
  }

  Widget _buildLoadFailed() {
    return GestureDetector(
      onTap: _clickRetry,
      child: this.failed,
    );
  }

  /// 点击重试，加载数据
  void _clickRetry() {
    if (this.onRetry != null) {
      this.onRetry();
    }
  }
}

/// 加载中状态
enum LoadingStatus {
  /// 加载中
  loading,

  /// 加载成功
  success,

  /// 加载失败
  failed,
}
